@@ -13,16 +13,15 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "module.h"
 #include "modules.h"
 #include "network.h"
 #include "net-sendbuffer.h"
-#include "line-split.h"
 #include "rawlog.h"
 #include "misc.h"
 
@@ -57,6 +56,8 @@ void irc_send_cmd_full(IRC_SERVER_REC *server, const char *cmd,
 		return;
 
 	len = strlen(cmd);
+	if (server->cmdcount == 0)
+		irc_servers_start_cmd_timeout();
 	server->cmdcount++;
 
 	if (!raw) {
@@ -83,20 +84,20 @@ void irc_send_cmd_full(IRC_SERVER_REC *server, const char *cmd,
 
 	if (send_now) {
                 irc_server_send_data(server, cmd, len);
-		return;
-	}
-
-	/* add to queue */
-	if (immediate) {
-		server->cmdqueue = g_slist_prepend(server->cmdqueue,
-						   server->redirect_next);
-		server->cmdqueue = g_slist_prepend(server->cmdqueue,
-						   g_strdup(cmd));
 	} else {
-		server->cmdqueue = g_slist_append(server->cmdqueue,
-						  g_strdup(cmd));
-		server->cmdqueue = g_slist_append(server->cmdqueue,
-						  server->redirect_next);
+
+		/* add to queue */
+		if (immediate) {
+			server->cmdqueue = g_slist_prepend(server->cmdqueue,
+							   server->redirect_next);
+			server->cmdqueue = g_slist_prepend(server->cmdqueue,
+							   g_strdup(cmd));
+		} else {
+			server->cmdqueue = g_slist_append(server->cmdqueue,
+							  g_strdup(cmd));
+			server->cmdqueue = g_slist_append(server->cmdqueue,
+							  server->redirect_next);
+		}
 	}
         server->redirect_next = NULL;
 }
@@ -203,7 +204,7 @@ void irc_send_cmd_split(IRC_SERVER_REC *server, const char *cmd,
 	tmp = nicklist;
 	for (;; tmp++) {
 		if (*tmp != NULL) {
-			g_string_sprintfa(nickstr, "%s,", *tmp);
+			g_string_append_printf(nickstr, "%s,", *tmp);
 			if (++count < max_nicks)
 				continue;
 		}
@@ -292,7 +293,7 @@ static void irc_server_event(IRC_SERVER_REC *server, const char *line,
         g_strdown(event);
 
         /* check if event needs to be redirected */
-	signal = server_redirect_get_signal(server, event, args);
+	signal = server_redirect_get_signal(server, nick, event, args);
 	if (signal == NULL)
 		signal = event;
         else
@@ -307,28 +308,6 @@ static void irc_server_event(IRC_SERVER_REC *server, const char *line,
 	g_free(event);
 }
 
-/* Read line from server */
-static int irc_receive_line(SERVER_REC *server, char **str, int read_socket)
-{
-	char tmpbuf[512];
-	int recvlen, ret;
-
-	g_return_val_if_fail(server != NULL, -1);
-	g_return_val_if_fail(str != NULL, -1);
-
-	recvlen = !read_socket ? 0 :
-		net_receive(net_sendbuffer_handle(server->handle),
-			    tmpbuf, sizeof(tmpbuf));
-
-	ret = line_split(tmpbuf, recvlen, str, &server->buffer);
-	if (ret == -1) {
-		/* connection lost */
-		server->connection_lost = TRUE;
-		server_disconnect(server);
-	}
-	return ret;
-}
-
 static char *irc_parse_prefix(char *line, char **nick, char **address)
 {
 	char *p;
@@ -384,6 +363,7 @@ static void irc_parse_incoming(SERVER_REC *server)
 {
 	char *str;
 	int count;
+	int ret;
 
 	g_return_if_fail(server != NULL);
 
@@ -391,9 +371,10 @@ static void irc_parse_incoming(SERVER_REC *server)
 	   too slowly, so read only a few times from the socket before
 	   letting other tasks to run. */
 	count = 0;
+	ret = 0;
 	server_ref(server);
 	while (!server->disconnected &&
-	       irc_receive_line(server, &str, count < MAX_SOCKET_READS) > 0) {
+	       (ret = net_sendbuffer_receive_line(server->handle, &str, count < MAX_SOCKET_READS)) > 0) {
 		rawlog_input(server->rawlog, str);
 		signal_emit_id(signal_server_incoming, 2, server, str);
 
@@ -402,6 +383,11 @@ static void irc_parse_incoming(SERVER_REC *server)
 
 		count++;
 	}
+	if (ret == -1) {
+		/* connection lost */
+		server->connection_lost = TRUE;
+		server_disconnect(server);
+	}
 	server_unref(server);
 }
 
